import junit.framework.TestCase;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URISyntaxException;
import ru.protek.component.xzip.XCRC32;
import ru.protek.component.xzip.XZipEntry;
import ru.protek.component.xzip.XZipInputStream;
import ru.protek.component.xzip.XZipOutputStream;

/**
 *    ,    ,
 *  .
 */
public class TestOne extends TestCase {

    /**
     *  CRC32.
     */
    public void testCRC32() {
        XCRC32 myCRC = new XCRC32();
        java.util.zip.CRC32 javaCRC = new java.util.zip.CRC32();
        javaCRC.reset();
        myCRC.reset();
        for (int i = 0; i < 8192; i++){
            javaCRC.update(i);
            myCRC.update(i);
            assertEquals(javaCRC.getValue(), myCRC.getValue());
        }
    }

    /**
     *    .    .
     * @throws IOException  -.
     * @throws URISyntaxException  URI.
     */
    public void testZip() throws IOException, URISyntaxException {
        One one = new One();
        one.zip();
    }

    /**
     *    .    .
     * @throws IOException  -.
     */
    public void testUnZip() throws IOException {
        One one = new One();
        one.unzip();
    }

    /**
     *  WinZipCrypt.zip    WinZip.exe.
     *     "vlad".
     *           .crypt.
     * @throws IOException  -.
     */
    public void testWinZipUnZip() throws IOException {
        XZipEntry ze;
        FileOutputStream fo;
        FileInputStream fi = new FileInputStream("WinZipCrypt.zip");
        XZipInputStream zi = new XZipInputStream(fi);
        zi.setPassword("vlad");
        zi.setPasswordEncode("Cp1256");
        for(ze = zi.getNextEntry(); ze != null; ze = zi.getNextEntry()) {
            fo = new FileOutputStream(ze.getName() + ".decrypt");
            for(int r = zi.read();r >= 0; r = zi.read()) fo.write(r);
            fo.close();
            zi.closeEntry();
        }
        zi.close();
    }

    /**
     *  DelphiCrypt.zip     Delphi VCLZip.
     *     "vlad".
     *           .crypt.
     * @throws IOException  -.
     */
    public void testDelphiUnZip() throws IOException {
        XZipEntry ze;
        FileOutputStream fo;
        FileInputStream fi = new FileInputStream("DelphiCrypt.zip");
        XZipInputStream zi = new XZipInputStream(fi);
        zi.setPassword("vlad");
        zi.setPasswordEncode("Cp1256");
        for(ze = zi.getNextEntry(); ze != null; ze = zi.getNextEntry()) {
            fo = new FileOutputStream(ze.getName() + ".decrypt");
            for(int r = zi.read();r >= 0; r = zi.read()) fo.write(r);
            fo.close();
            zi.closeEntry();
        }
        zi.close();
    }

    /**
     *      .
     * @param in  .
     * @param out  .
     * @throws IOException  -.
     */
    private void fillStreamFromStream(InputStream in, OutputStream out) throws IOException {
        int c;
        while ((c = in.read()) != -1) {
            out.write(c);
        }
    }

    /**
     *     .
     *     .
     * @param in1  .
     * @param in2  .
     * @throws IOException  -.
     */
    private void compareTwoStreams(InputStream in1, InputStream in2) throws IOException {
        int c1;
        int c2;
        do {
            c1 = in1.read();
            c2 = in2.read();
            if (c1 != c2) fail("     !");
        } while (c1 != -1);
    }

    /**
     *  3  (test1.xml, test2.xml, test3.xsd)   crypt.zip c
     *  "vlad".
     * @throws IOException  -.
     */
    public void testZipWithPassword() throws IOException {
        FileInputStream fi;

        FileOutputStream fo = new FileOutputStream("crypt.zip");
        XZipOutputStream zo = new XZipOutputStream(fo);
        zo.setPassword("vlad");
        zo.setPasswordEncode("Cp1256");

        fi = new FileInputStream("test1.xml");
        zo.putNextEntry(new XZipEntry("test1.xml"));
        fillStreamFromStream(fi, zo);

        fi.close();
        fi = new FileInputStream("test2.xml");
        zo.putNextEntry(new XZipEntry("test2.xml"));
        fillStreamFromStream(fi, zo);
        fi.close();

        fi = new FileInputStream("test3.xsd");
        zo.putNextEntry(new XZipEntry("test3.xsd"));
        fillStreamFromStream(fi, zo);
        fi.close();

        zo.close();

    }

    /**
     *     "vlad"  crypt.zip  
     * (test1.xml, test2.xml, test3.xsd).     .decrypt.
     * @throws IOException  -.
     */
    public void testUnZipWithPassword() throws IOException {
        XZipEntry ze;
        FileOutputStream fo;
        FileInputStream fi = new FileInputStream("crypt.zip");
        XZipInputStream zi = new XZipInputStream(fi);
        zi.setPassword("vlad");
        zi.setPasswordEncode("Cp1256");
        for(ze = zi.getNextEntry(); ze != null; ze = zi.getNextEntry()) {
            fo = new FileOutputStream(ze.getName() + ".decrypt");
            for(int r = zi.read();r >= 0; r = zi.read()) fo.write(r);
            fo.close();
            zi.closeEntry();
        }
        zi.close();
    }

    /**
     *  3  (test1.xml, test2.xml, test3.xsd)   crypt.zip c
     *  "vlad".
     *     "vlad"  crypt.zip  
     * (test1.xml, test2.xml, test3.xsd).     .decrypt.
     *    (test1.xml, test2.xml, test3.xsd)
     *  (test1.xml.decrypt, test2.xml.decrypt, test3.xsd.decrypt).  
     *   .
     * @throws IOException  -.
     */
    public void testZipAndUnZipWithPassword() throws IOException {
        FileInputStream fi1;
        FileInputStream fi2;

        testZipWithPassword();
        testUnZipWithPassword();

        // 
        fi1 = new FileInputStream("test1.xml");
        fi2 = new FileInputStream("test1.xml.decrypt");
        compareTwoStreams(fi1, fi2);
        fi1.close();
        fi2.close();

        fi1 = new FileInputStream("test2.xml");
        fi2 = new FileInputStream("test2.xml.decrypt");
        compareTwoStreams(fi1, fi2);
        fi1.close();
        fi2.close();

        fi1 = new FileInputStream("test3.xsd");
        fi2 = new FileInputStream("test3.xsd.decrypt");
        compareTwoStreams(fi1, fi2);
        fi1.close();
        fi2.close();
        //
    }

    /**
     *  3  (test1.xml, test2.xml, test3.xsd)   crypt.zip c
     *  {100, 35, 50, 111, -56, -90, 40, 12, -8, 10}.
     * @throws IOException  -.
     */
    public void testZipWithBytePassword() throws IOException {
        FileInputStream fi;

        FileOutputStream fo = new FileOutputStream("crypt.zip");
        XZipOutputStream zo = new XZipOutputStream(fo);

        byte[] bytePass = {100, 35, 50, 111, -56, -90, 40, 12, -8, 10};
        zo.setPasswordAsBytes(bytePass);

        fi = new FileInputStream("test1.xml");
        zo.putNextEntry(new XZipEntry("test1.xml"));
        fillStreamFromStream(fi, zo);

        fi.close();
        fi = new FileInputStream("test2.xml");
        zo.putNextEntry(new XZipEntry("test2.xml"));
        fillStreamFromStream(fi, zo);
        fi.close();

        fi = new FileInputStream("test3.xsd");
        zo.putNextEntry(new XZipEntry("test3.xsd"));
        fillStreamFromStream(fi, zo);
        fi.close();

        zo.close();

    }

    /**
     *    
     * {100, 35, 50, 111, -56, -90, 40, 12, -8, 10}
     *  crypt.zip  
     * (test1.xml, test2.xml, test3.xsd).     .decrypt.
     * @throws IOException  -.
     */
    public void testUnZipWithBytePassword() throws IOException {
        XZipEntry ze;
        FileOutputStream fo;
        FileInputStream fi = new FileInputStream("crypt.zip");
        XZipInputStream zi = new XZipInputStream(fi);
        byte[] bytePass = {100, 35, 50, 111, -56, -90, 40, 12, -8, 10};
        zi.setPasswordAsBytes(bytePass);
        for(ze = zi.getNextEntry(); ze != null; ze = zi.getNextEntry()) {
            fo = new FileOutputStream(ze.getName() + ".decrypt");
            for(int r = zi.read();r >= 0; r = zi.read()) fo.write(r);
            fo.close();
            zi.closeEntry();
        }
        zi.close();
    }

    /**
     *  3  (test1.xml, test2.xml, test3.xsd)   crypt.zip c
     *  {100, 35, 50, 111, -56, -90, 40, 12, -8, 10}.
     *    
     * {100, 35, 50, 111, -56, -90, 40, 12, -8, 10}
     *  crypt.zip  
     * (test1.xml, test2.xml, test3.xsd).     .decrypt.
     *    (test1.xml, test2.xml, test3.xsd)
     *  (test1.xml.decrypt, test2.xml.decrypt, test3.xsd.decrypt).  
     *   .
     * @throws IOException  -.
     */
    public void testZipAndUnZipWithBytePassword() throws IOException {
        FileInputStream fi1;
        FileInputStream fi2;

        testZipWithBytePassword();
        testUnZipWithBytePassword();

        // 
        fi1 = new FileInputStream("test1.xml");
        fi2 = new FileInputStream("test1.xml.decrypt");
        compareTwoStreams(fi1, fi2);
        fi1.close();
        fi2.close();

        fi1 = new FileInputStream("test2.xml");
        fi2 = new FileInputStream("test2.xml.decrypt");
        compareTwoStreams(fi1, fi2);
        fi1.close();
        fi2.close();

        fi1 = new FileInputStream("test3.xsd");
        fi2 = new FileInputStream("test3.xsd.decrypt");
        compareTwoStreams(fi1, fi2);
        fi1.close();
        fi2.close();
        //
    }

}
